@@ -4,6 +4,7 @@ modules.define(
     // Required modules...
     'CommonHelpers',
     'CommonModal',
+    'CommonNotify',
     'CompareProxyDialogModal',
     'CompareRowClick',
     'CompareRowsHelpers',
@@ -13,6 +14,7 @@ modules.define(
     // Resolved modules...
     CommonHelpers,
     CommonModal,
+    CommonNotify,
     CompareProxyDialogModal,
     CompareRowClick,
     CompareRowsHelpers,
@@ -45,24 +47,9 @@ modules.define(
 
       // Methods...
 
-      shiftRowHandler(event, row, row_id) {
-        CompareRowClick.disableRowClickHandler();
-        // Add row from source to target array
-        event.preventDefault();
-        row.parentElement.parentElement.classList.add('shift-right');
-        document.getElementById('replace-with-target-arrow').style.display = 'none';
-        const { source_data, target_data } = this.sharedData;
-        const obj = source_data.find((item) => item.row_id == row_id);
-        // Creating new object with unique (?) row_id
-        const newObj = { ...obj, row_id: 'source-' + obj.row_id };
-        target_data.push(newObj);
-        this.sortTable(target_data);
-        this.buildTable('target-table', target_data, true);
-        this.sharedData.comment += `* Added source exchange of ${obj.amount} ${obj.unit} ${obj.name} in ${obj.location}.\n`;
-        row.parentElement.innerHTML = `<i class="fa-solid fa-check"></i>`;
-      },
-
       buildRow(data, is_target) {
+        // NOTE: Row operations handlers: shiftRowHandler, editNumberHandler, expandRowHandler, removeRowHandler
+        // Reg: \<\(shiftRowHandler\|editNumberHandler\|expandRowHandler\|removeRowHandler\>\)
         const { collapsedRows } = CompareRowsHelpers;
         const {
           collapsed,
@@ -74,6 +61,7 @@ modules.define(
           amount,
           name,
           input_id,
+          matched,
         } = data;
         const rowKind = is_target ? 'target' : 'source';
         const collapsedId = CompareRowsHelpers.getCollapsedId(rowKind, rowId);
@@ -111,19 +99,23 @@ modules.define(
           isCollapsed && CompareRowsHelpers.buildCollapsedHandlerRow(rowKind, rowId, data);
         // Create class name...
         const className = [isCollapsed && 'collapsed'].filter(Boolean).join(' ');
-        const attrs = [['class', className], isCollapsed && ['collapsed-id', collapsedId]]
+        const trAttrs = [['class', className], isCollapsed && ['collapsed-id', collapsedId]]
           .filter(Boolean)
           .map(([name, value]) => value && name + '="' + CommonHelpers.quoteHtmlAttr(value) + '"')
           .filter(Boolean)
           .join(' ');
         const start = `<tr
-          ${attrs}
+          ${trAttrs}
           row_id="${rowId}"
           onClick="CompareCore.clickRowHandler(this)"
         >`;
-        const end = `<td class="cell-name"><div><a
-            onClick="CompareCore.disableRowClickHandler(this)"
-            href="${url}">${name}</a></div></td>
+        // Issue #59: Name cell with check icon for 'matched' rows...
+        const nameClassName = `cell-name${matched && ' matched'}`;
+        let nameContent = `<a onClick="CompareCore.disableRowClickHandler(this)" href="${url}">${name}</a>`;
+        if (!is_target && matched) {
+          nameContent = `<i class="name-icon fa-solid fa-check"></i><span class="name-text">${nameContent}</span>`;
+        }
+        const end = `<td class="${nameClassName}"><div>${nameContent}</div></td>
           <td class="cell-location"><div>${location}</div></td>
           <td class="cell-unit"><div>${unit}</div></td>
         </tr>`;
@@ -216,7 +208,7 @@ modules.define(
       },
 
       replaceWithTargetHandler(elem) {
-        CompareRowClick.disableRowClickHandler();
+        // CompareRowClick.disableRowClickHandler(); // NOTE: It's not inside the row
         const { target_node, target_data } = this.sharedData;
         // Trying to make unique row_id...
         const uniqueCounter = ++this.targetNodesCounter;
@@ -227,14 +219,48 @@ modules.define(
         this.sharedData.comment += `* Collapsed input exchanges to target node\n`;
         // TODO: Is sorting required here?
         this.buildTable('target-table', target_data, true);
-        elem.innerHTML = '';
+        // Hide icon...
+        elem.classList.toggle('hidden', true); // Old code: `innerHTML = ''`
+      },
+
+      /** setModifiedRowsStatus -- Set modified status: if it had set then `replaceWithTargetHandler` operarion should be disabled
+       * @param {boolean} [hasModifiedRows]
+       */
+      setModifiedRowsStatus(hasModifiedRows) {
+        /* // XXX Do we need this global state?
+         * const rootNode = CompareRowsHelpers.getRootNode();
+         * rootNode.classList.toggle('has-modified-rows', hasModifiedRows);
+         */
+        const replaceButton = document.getElementById('replace-with-target-arrow');
+        replaceButton.classList.toggle('disabled', hasModifiedRows);
+      },
+
+      // Row operations handlers: shiftRowHandler, expandRowHandler,
+      // removeRowHandler, editNumberHandler (below, in 'Edit number modal
+      // dialog' section, actual final handler: setNumberAndCloseModalHandler)...
+
+      shiftRowHandler(event, row, row_id) {
+        CompareRowClick.disableRowClickHandler();
+        // Add row from source to target array
+        event.preventDefault();
+        const parentNode = row.parentElement;
+        parentNode.parentElement.classList.add('shift-right');
+        parentNode.innerHTML = `<i class="fa-solid fa-check"></i>`;
+        const { source_data, target_data } = this.sharedData;
+        const obj = source_data.find((item) => item.row_id == row_id);
+        // Creating new object with unique (?) row_id
+        const newObj = { ...obj, row_id: 'source-' + obj.row_id };
+        target_data.push(newObj);
+        this.sortTable(target_data);
+        this.buildTable('target-table', target_data, true);
+        this.sharedData.comment += `* Added source exchange of ${obj.amount} ${obj.unit} ${obj.name} in ${obj.location}.\n`;
+        this.setModifiedRowsStatus(true);
       },
 
       removeRowHandler(element) {
         CompareRowClick.disableRowClickHandler();
         const rowId = element.closest('td').getAttribute('row_id');
         const { target_data } = this.sharedData;
-        document.getElementById('replace-with-target-arrow').style.display = 'none';
         target_data.filter((obj, index, arr) => {
           if (obj.row_id == rowId) {
             this.sharedData.comment += `* Removed exchange of ${obj.amount} ${obj.unit} ${obj.name} from ${obj.location}.\n`;
@@ -244,6 +270,7 @@ modules.define(
           return false;
         });
         this.buildTable('target-table', target_data, true);
+        this.setModifiedRowsStatus(true);
       },
 
       expandRowHandler(element) {
@@ -253,15 +280,16 @@ modules.define(
         const elAmount = element.getAttribute('amount');
         const url = '/expand/' + elInputId + '/' + elAmount + '/';
         const node = target_data.find((item) => item.input_id == elInputId);
-
-        document.getElementById('replace-with-target-arrow').style.display = 'none';
         this.sharedData.comment += `* Expanded process inputs of ${node.amount} ${node.unit} from ${node.name} in ${node.location}.\n`;
         fetch(url)
-          .then((response) => {
-            if (!response.ok) {
-              throw new Error(`HTTP error ${response.status}`);
+          .then((res) => {
+            if (!res.ok) {
+              const error = new Error(
+                `Can't load url '${res.url}': ${res.statusText}, ${res.status}`,
+              );
+              throw error;
             }
-            return response.json();
+            return res.json();
           })
           .then((data) => {
             const { target_data } = this.sharedData;
@@ -272,19 +300,35 @@ modules.define(
             });
             this.sortTable(target_data);
             this.removeRowHandler(element);
+            this.setModifiedRowsStatus(true);
+          })
+          .catch((error) => {
+            // eslint-disable-next-line no-console
+            console.error('[CompareCore:copyToClipboardHandler] Catched error', error);
+            // eslint-disable-next-line no-debugger
+            debugger;
+            // Show error in the notification toast...
+            CommonNotify.showError(`Can't expand row: ${CommonHelpers.getErrorText(error)}`);
           });
       },
 
       // Edit number modal dialog (TODO: Extract all this code into it's own dedicated module?)
 
-      replaceAmountRowHandler(elem, target_id) {
+      replaceAmountRowHandler(elem) {
         CompareRowClick.disableRowClickHandler();
-        const s = this.sharedData.source_data.find(
-          (item) => item.row_id == elem.getAttribute('source_id'),
-        );
-        const t = this.sharedData.target_data.find((item) => item.row_id == target_id);
-        this.sharedData.comment += `* Used source database amount ${s.amount} ${s.unit} from ${s.name} in ${s.location} instead of ${t.amount} ${t.unit} from ${t.name} in ${t.location}.\n`;
-        document.getElementById('number-current-amount').innerText = elem.getAttribute('amount');
+        const target_id = elem.getAttribute('target_id');
+        const source_id = elem.getAttribute('source_id');
+        const sourceRow = this.sharedData.source_data.find((item) => item.row_id == source_id);
+        if (!sourceRow) {
+          throw new Error('Not found source row for id: ' + source_id);
+        }
+        const targetRow = this.sharedData.target_data.find((item) => item.row_id == target_id);
+        if (!targetRow) {
+          throw new Error('Not found target row for id: ' + target_id);
+        }
+        this.sharedData.comment += `* Used source database amount ${sourceRow.amount} ${sourceRow.unit} from ${sourceRow.name} in ${sourceRow.location} instead of ${targetRow.amount} ${targetRow.unit} from ${targetRow.name} in ${targetRow.location}.\n`;
+        const amount = sourceRow.amount;
+        document.getElementById('number-current-amount').innerText = amount;
       },
 
       rescaleAmountHandler(event) {
@@ -314,7 +358,7 @@ modules.define(
         document.getElementById('number-current-amount').innerText = new_value;
       },
 
-      setNumberHandler(event) {
+      setNumberAndCloseModalHandler(event) {
         event.preventDefault();
         const { target } = event;
         const formRootElem = target.closest('.set-number-modal-form');
@@ -325,12 +369,13 @@ modules.define(
         const item = target_data.find(({ row_id }) => row_id === rowId);
         if (item) {
           item.amount = current;
+          // TODO: Number formatting is weird here. It would be nice to fix it.
           item.amount_display = current.toExponential();
         }
-        // TODO: Number formatting is weird here. It would be nice to fix it.
         CommonModal.hideModal();
         this.sortTable(target_data);
         this.buildTable('target-table', target_data, true);
+        this.setModifiedRowsStatus(true);
       },
 
       resetNumberHandler(event) {
@@ -351,17 +396,13 @@ modules.define(
         inputNumber.value = amount;
       },
 
-      editNumberHandler(link) {
-        CompareRowClick.disableRowClickHandler();
-        const { source_data, target_data } = this.sharedData;
-        const td = link.closest('td');
-        const rowId = td.getAttribute('row_id');
-        const row = target_data.find(({ row_id }) => row_id == rowId);
-
-        let start = `
+      getEditNumberModalContent(row) {
+        const rowId = row.row_id;
+        const { source_data } = this.sharedData;
+        const start = `
           <div class="set-number-modal-table">
             <p>Click on a row to take that value</p>
-            <table id="edit-number-table" class="fixed-table modal-table" width="100%">
+            <table id="edit-number-table" class="fixed-table fixed-table-active modal-table" width="100%">
               <thead>
                 <tr>
                   <th>Amount</th>
@@ -371,17 +412,15 @@ modules.define(
               </thead>
               <tbody>
         `;
-
-        source_data.forEach((item) => {
-          start += `
-                <tr amount="${item.amount}" source_id="${item.row_id}" onClick="CompareCore.replaceAmountRowHandler(this, ${row.row_id})">
-                  <td><div>${item.amount_display}</div></td>
-                  <td><div>${item.name}</div></td>
-                  <td><div>${item.unit}</div></td>
-                </tr>
+        const rowsContent = source_data.map((item) => {
+          return `
+            <tr source_id="${item.row_id}" target_id="${rowId}" onClick="CompareCore.replaceAmountRowHandler(this)">
+              <td><div>${item.amount_display}</div></td>
+              <td><div>${item.name}</div></td>
+              <td><div>${item.unit}</div></td>
+            </tr>
           `;
         });
-
         const end = `
               <tbody>
             </table>
@@ -390,7 +429,7 @@ modules.define(
             <div class="strong"><strong>Original amount:</strong> ${row.amount}</div>
             <div class="strong"><strong>Current amount:</strong> <span id="number-current-amount">${row.amount}</span></div>
             <hr />
-            <button class="button-primary" id="close-number-editor">Set and close</button>
+            <button class="button-primary" id="set-and-close-number-editor">Set and close</button>
             <button id="reset-number">Reset number</button>
             <hr />
             <form>
@@ -408,9 +447,18 @@ modules.define(
             </form>
           </div>
         `;
+        return start + rowsContent.join('') + end;
+      },
+
+      editNumberHandler(link) {
+        CompareRowClick.disableRowClickHandler();
+        const { target_data } = this.sharedData;
+        const td = link.closest('td');
+        const rowId = td.getAttribute('row_id');
+        const row = target_data.find(({ row_id }) => row_id == rowId);
 
         const title = [row.name, row.location, row.unit].filter(Boolean).join(' | ');
-        const content = start + end;
+        const content = this.getEditNumberModalContent(row);
 
         CommonModal.setModalContentId('set-number-modal')
           .setTitle(title)
@@ -430,8 +478,8 @@ modules.define(
           .getElementById('new-number-button')
           .addEventListener('click', this.setNewNumberHandler.bind(this));
         document
-          .getElementById('close-number-editor')
-          .addEventListener('click', this.setNumberHandler.bind(this));
+          .getElementById('set-and-close-number-editor')
+          .addEventListener('click', this.setNumberAndCloseModalHandler.bind(this));
         document
           .getElementById('reset-number')
           .addEventListener('click', this.resetNumberHandler.bind(this));
@@ -441,6 +489,37 @@ modules.define(
         event.preventDefault();
       },
 
+      // Copy to clipboard
+
+      copyToClipboardHandler(node) {
+        const { sharedData } = this;
+        const nodeType = node.getAttribute('data-node-type');
+        const nameId = nodeType + '_node_name'; // 'source_node_name' | 'target_node_name'
+        const urlId = nodeType + '_node_url'; // 'source_node_url' | 'target_node_url'
+        const name = sharedData[nameId];
+        const url = sharedData[urlId];
+        const text = '[' + name + '](' + url + ')';
+        // NOTE: Docment should be focused...
+        navigator.clipboard
+          .writeText(text)
+          .then(() => {
+            // Show notification...
+            CommonNotify.showSuccess('Text already copied to clipboard');
+          })
+          .catch((error) => {
+            // eslint-disable-next-line no-console
+            console.error('[CompareCore:copyToClipboardHandler] Catched error', error);
+            // eslint-disable-next-line no-debugger
+            debugger;
+            // Show error in the notification toast...
+            CommonNotify.showError(
+              `Can't copy text to clipboard: ${CommonHelpers.getErrorText(error)}`,
+            );
+          });
+        // TODO: To catch promise resolve or catch?
+        return false;
+      },
+
       // Start...
 
       /** start -- Initialize compare feature (entry point)
@@ -452,9 +531,11 @@ modules.define(
         CompareRowsHelpers.sharedData = sharedData;
         CompareProxyDialogModal.sharedData = sharedData;
 
-        const { source_data, target_data } = this.sharedData;
+        /* // DEBUG: Show demo notifiers...
+         * CommonNotify.showDemo();
+         */
 
-        // console.log('[CompareCore:start] sharedData', sharedData);
+        const { source_data, target_data } = this.sharedData;
 
         // Prepare all the tables data...
         this.indexTable(source_data);
