@@ -10,12 +10,18 @@ import bw2data as bd
 import flask
 import tomli
 import whoosh
+from bw2analyzer import PageRank
 from bw2data.backends import ActivityDataset as AD
 from flask_httpauth import HTTPBasicAuth
 from peewee import fn
 from werkzeug.security import check_password_hash, generate_password_hash
 
-from .utils import get_match_types, name_close_enough, similar_location
+from .utils import (
+    MATCH_TYPE_ABBREVIATIONS,
+    get_match_types,
+    name_close_enough,
+    similar_location,
+)
 
 BASE_DIR = Path(__file__).resolve().parent
 DATA_DIR = BASE_DIR / "data"
@@ -44,12 +50,17 @@ def configure_app(filepath, app):
         with open(config_fp, "rb") as f:
             config = tomli.load(f)
         assert "users" in config
+        assert "configs" in config
+        assert "roles" in config
+        assert "files" in config
     except:
         # NOTE: If you got this exception, make sure first you don't have
         # windows-style backslahes in your config file for file names (no
         # double `\\`).
         raise ValueError("Invalid or unreadable config file")
 
+    app.config["mb_configs"] = config["configs"]
+    app.config["mb_roles"] = config["roles"]
     app.config["mb_users"] = {
         user: generate_password_hash(password)
         for user, password in config["users"].items()
@@ -66,185 +77,127 @@ def configure_app(filepath, app):
     return app
 
 
-def get_context():
-    project = flask.request.cookies.get("project")
-    if not project:
-        return flask.redirect(flask.url_for("select_project"))
-    source = flask.request.cookies.get("source")
-    target = flask.request.cookies.get("target")
-    if not (source and target):
-        return flask.redirect(flask.url_for("select_databases"))
-    proxy = flask.request.cookies.get("proxy")
-    return project, source, target, proxy
+def return_invalid_config(config):
+    # TBD #90
+    raise ValueError(f"Invalid configuration: {config}")
 
 
-def get_files():
-    files = flask.request.cookies.get("files")
-    if files:
-        files = json.loads(files)
+def get_role(config):
+    for role, lst in config["roles"].items():
+        for name in lst:
+            if name == config["user"]:
+                return role
+
+
+def get_config():
+    config_label = flask.request.cookies.get("config")
+    if not config_label:
+        return flask.redirect(flask.url_for("select_config"))
+
+    try:
+        # TBD #90: Provide good error messages
+        config = matchbox_app.config["mb_configs"][config_label]
+        config["roles"] = matchbox_app.config["mb_roles"]
+        config["config_label"] = config_label
+        config["user"] = auth.current_user()
+        config["role"] = get_role(config)
+        config["output_dir"] = matchbox_app.config["mb_output_dir"]
+    except KeyError:
+        return flask.redirect(flask.url_for("select_config"))
+
+    if config["project"] not in bd.projects:
+        return_invalid_config(config)
+
+    bd.projects.set_current(config["project"])
+    return config
+
+
+def get_match_type_for_source_process(node):
+    if hasattr(node, "data"):
+        if "match_type" in node.data:
+            label = matchbox_app.config["mb_match_types"].get(
+                node.data["match_type"], "Unknown"
+            )
+            return MATCH_TYPE_ABBREVIATIONS.get(label, label)
+        elif node.data.get("proxy_id"):
+            mt = bd.get_node(id=node.data["proxy_id"]).get("match_type")
+            label = matchbox_app.config["mb_match_types"].get(mt, "Unknown")
+            return MATCH_TYPE_ABBREVIATIONS.get(label, label)
+        else:
+            return "Unknown"
+    elif "match_type" in node:
+        label = matchbox_app.config["mb_match_types"].get(node["match_type"], "Unknown")
+        return MATCH_TYPE_ABBREVIATIONS.get(label, label)
     else:
-        included = filter(lambda x: x.suffix.lower() == ".json", DATA_DIR.iterdir())
-        files = [
-            {
-                "index": str(index),
-                "filename": path.name,
-                "dirpath": str(path.parent),
-                "writable": os.access(path, os.W_OK),
-                "enabled": True,
-            }
-            for index, path in enumerate(included)
-        ]
-    return files
+        mt = bd.get_node(id=node["proxy_id"]).get("match_type")
+        if not mt or mt == "0":
+            return "Unknown"
+        else:
+            label = matchbox_app.config["mb_match_types"].get(mt, "Unknown")
+            return MATCH_TYPE_ABBREVIATIONS.get(label, label)
 
 
-@matchbox_app.route("/project", methods=["POST", "GET"])
+@matchbox_app.route("/page-rank", methods=["GET"])
 @auth.login_required
-def select_project():
-    files = get_files()
+def build_page_rank_cache():
+    config = get_config()
+    if config["role"] != "editors":
+        flask.abort(403)
+    pr = PageRank(bd.Database(config["source"])).calculate()
+    with open(config["output_dir"] / "page-rank.json", "w") as f:
+        json.dump({"scores": pr}, f, indent=2)
+    return ""
 
+
+def get_page_rank_cache():
+    fp = Path(matchbox_app.config["mb_output_dir"]) / "page-rank.json"
+    if fp.is_file():
+        return json.load(open(fp))["scores"]
+    else:
+        return []
+
+
+@matchbox_app.route("/select-config", methods=["POST", "GET"])
+@auth.login_required
+def select_config():
     if flask.request.method == "POST":
-        project = flask.request.form["project"]
-        resp = flask.make_response(flask.redirect(flask.url_for("select_databases")))
-        resp.set_cookie("project", project)
+        config_name = flask.request.form["config"]
+        resp = flask.make_response(flask.redirect(flask.url_for("index")))
+        resp.set_cookie("config", config_name)
         return resp
     else:
         resp = flask.make_response(
             flask.render_template(
-                "project.html",
-                file_number=sum(1 for obj in files if obj["enabled"]),
-                projects=[o for o in bd.projects],
-                user=auth.current_user(),
+                "select_config.html",
+                config_labels=matchbox_app.config["mb_configs"],
+                config={"user": auth.current_user()},
             )
         )
-        resp.delete_cookie("project")
-        resp.delete_cookie("source")
-        resp.delete_cookie("target")
+        resp.delete_cookie("config")
         return resp
 
 
-@matchbox_app.route("/databases", methods=["POST", "GET"])
-@auth.login_required
-def select_databases():
-    project = flask.request.cookies.get("project")
-    if not project:
-        return flask.redirect(flask.url_for("select_project"))
-    bd.projects.set_current(project)
-
-    files = get_files()
-
-    if flask.request.method == "POST":
-        source = flask.request.form["source"]
-        target = flask.request.form["target"]
-        proxy_existing = flask.request.form["proxy-existing"]
-        proxy_new = flask.request.form["proxy-new"].strip()
-
-        if source != target:
-            resp = flask.make_response(flask.redirect(flask.url_for("index")))
-            resp.set_cookie("source", source)
-            resp.set_cookie("target", target)
-
-            if proxy_new:
-                bd.Database(proxy_new).register()
-                resp.set_cookie("proxy", proxy_new)
-            else:
-                resp.set_cookie("proxy", proxy_existing)
-
-            return resp
-
-    resp = flask.make_response(
-        flask.render_template(
-            "databases.html",
-            project=project,
-            file_number=sum(1 for obj in files if obj["enabled"]),
-            user=auth.current_user(),
-            databases=bd.databases,
-        )
-    )
-    resp.delete_cookie("source")
-    resp.delete_cookie("target")
-    return resp
-
-
-@matchbox_app.route("/files", methods=["POST", "GET"])
-@auth.login_required
-def select_matching_files():
-    context = get_context()
-    if isinstance(context, flask.Response):
-        proj, s, t, proxy = None, None, None, None
-    else:
-        proj, s, t, proxy = context
-
-    files = get_files()
-
-    if flask.request.method == "POST":
-        # Don't get unchecked elements in forms
-        for line in files:
-            line["enabled"] = False
-
-        for key, value in flask.request.form.items():
-            index = key.replace("enabled-", "")
-            for line in files:
-                if line["index"] == index:
-                    line["enabled"] = value == "on"
-
-            resp = flask.make_response(flask.redirect(flask.url_for("index")))
-            resp.set_cookie("files", json.dumps(files))
-            return resp
-
-    # Format file cookie data for nicer form
-    files_formatted = [
-        sorted(
-            [obj for obj in files if obj["dirpath"] == dirpath],
-            key=lambda x: x["filename"],
-        )
-        for dirpath in sorted({obj["dirpath"] for obj in files})
-    ]
-
-    resp = flask.make_response(
-        flask.render_template(
-            "select_files.html",
-            files=files_formatted,
-            file_number=sum(1 for obj in files if obj["enabled"]),
-            project=proj,
-            proxy=proxy,
-            source=s,
-            target=t,
-            user=auth.current_user(),
-        )
-    )
-    resp.set_cookie("files", json.dumps(files))
-    return resp
-
-
 @matchbox_app.route("/", methods=["GET"])
 @auth.login_required
 def index():
-    context = get_context()
-    if isinstance(context, flask.Response):
-        return context
-    proj, s, t, proxy = context
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
 
-    files = get_files()
-
-    bd.projects.set_current(proj)
     return flask.render_template(
         "index.html",
-        title="bw_matchbox Index Page",
+        title="Matchbox Index",
         unmatched_link=True,
-        project=proj,
-        source=s,
-        target=t,
-        file_number=sum(1 for obj in files if obj["enabled"]),
-        table_data=[obj for obj, _ in zip(bd.Database(s), range(50))],
+        config=config,
+        table_data=[],
         query_string="",
-        database=s,
-        proxy=proxy,
-        user=auth.current_user(),
     )
 
 
 def apply_filter_to_qs(qs, filter_arg):
-    if filter_arg == "unmatched":
+    if not filter_arg:
+        return qs
+    elif filter_arg == "unmatched":
         return [node for node in qs if not node.data.get("matched")]
     else:
         return [node for node in qs if node.data.get(filter_arg)]
@@ -304,18 +257,19 @@ def processes():
         ]
     ```
     """
-    context = get_context()
-    proj, s, t, proxy = context
-    bd.projects.set_current(proj)
+    config = get_config()
 
-    database_label = flask.request.args.get("database") or s
+    database_label = flask.request.args.get("database") or config["source"]
     limit = maybe_int(flask.request.args.get("limit"))
     offset = maybe_int(flask.request.args.get("offset"))
     filter_arg = flask.request.args.get("filter")
+    order_by = flask.request.args.get("order_by")
+
+    pr = get_page_rank_cache()
 
     search_query = flask.request.args.get("search")
     if search_query:
-        # Override default very small search limit
+        # Override default too small search limit
         qs = [
             x._document
             for x in bd.Database(database_label).search(search_query, limit=1000)
@@ -324,10 +278,17 @@ def processes():
             qs = apply_filter_to_qs(qs, filter_arg)
         total_records = len(qs)
         qs = apply_limit_offset(qs, limit, offset)
+    elif order_by == "importance":
+        limit = limit or 250
+        offset = offset or 0
+        qs = [bd.get_node(id=id_)._document for _, id_ in pr[offset : offset + limit]]
+        print(limit, offset, len(qs))
+        qs = apply_filter_to_qs(qs, filter_arg)
+        print(limit, offset, len(qs))
+        total_records = len(bd.Database(config["source"]))
     else:
         qs = AD.select().where(AD.database == database_label)
 
-        order_by = flask.request.args.get("order_by")
         if order_by and order_by in ("name", "location", "product"):
             qs = qs.order_by(getattr(AD, order_by))
         else:
@@ -365,126 +326,55 @@ def processes():
     return flask.jsonify(payload)
 
 
-mwmarortalf = "Match with market and replace or remove transport and loss factor"
-
-MATCH_TYPE_ABBREVIATIONS = {
-    "No direct match available": "No direct match",
-    "Direct match with near-identical data": "Near-identical data",
-    "Match with updated data": "Updated data",
-    "Match with updated data and adding source transport": "Updated data w/ src trans",
-    "Match with market and replace or remove transport": "Market w/ diff trans",
-    mwmarortalf: "Market w/ diff trans loss",
-    "Equivalent datasets after modification": "Modifications",
-    "Replace aggregated with unit process": "Replace aggregated",
-    "Rescaled direct correspondence": "Rescaled",
-    "Similar data but different location": "New location",
-    "Unknown": "Unknown",
-}
-
-
-def get_match_type_for_source_process(node):
-    if hasattr(node, "data"):
-        if "match_type" in node.data:
-            label = matchbox_app.config["mb_match_types"].get(
-                node.data["match_type"], "Unknown"
-            )
-            return MATCH_TYPE_ABBREVIATIONS.get(label, label)
-        elif node.data.get("proxy_id"):
-            mt = bd.get_node(id=node.data["proxy_id"]).get("match_type")
-            label = matchbox_app.config["mb_match_types"].get(mt, "Unknown")
-            return MATCH_TYPE_ABBREVIATIONS.get(label, label)
-        else:
-            return "Unknown"
-    elif "match_type" in node:
-        label = matchbox_app.config["mb_match_types"].get(node["match_type"], "Unknown")
-        return MATCH_TYPE_ABBREVIATIONS.get(label, label)
-    else:
-        mt = bd.get_node(id=node["proxy_id"]).get("match_type")
-        if not mt or mt == "0":
-            return "Unknown"
-        else:
-            label = matchbox_app.config["mb_match_types"].get(mt, "Unknown")
-            return MATCH_TYPE_ABBREVIATIONS.get(label, label)
-
-
-# @matchbox_app.route("/match-status", methods=["GET"])
-# @auth.login_required
-# def match_status():
-#     crud = {}
-#     status = {}
-#     failed = []
-
-#     for filename in filter(lambda x: x.suffix.lower() == ".json", DATA_DIR.iterdir()):
-#         try:
-#             file_status = {'count': 0, 'duplicates': []}
-#             data = json.load(open(filename))
-#             for key in OPERATIONS:
-#                 for elem in data.get(key, []):
-#                     lookup = frozendict(elem['source'])
-#                     value = frozendict(elem['target'])
-#                     file_status['count'] += 1
-#                     if lookup in crud and crud.get(lookup) != value:
-#                         error = "Source: {}. Already have: {}. Given: {}".format(
-#                            dict(lookup), dict(crud[lookup]), dict(value))
-#                         if error not in file_status['duplicates']:
-#                             file_status['duplicates'].append(error)
-#                     else:
-#                         crud[lookup] = value
-#             status[filename.name] = file_status
-#         except:
-#             failed.append(filename.name)
-#             raise
-#     return flask.render_template(
-#         "match-status.html",
-#         failed=failed,
-#         status=[(key, value) for key, value in status.items()]
-#    )
-
-
 @matchbox_app.route("/search/", methods=["GET"])
 @auth.login_required
 def search():
-    context = get_context()
-    if isinstance(context, flask.Response):
-        return context
-    proj, s, t, proxy = context
-
-    files = get_files()
-    bd.projects.set_current(proj)
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
 
     q = flask.request.args.get("q")
     embed = flask.request.args.get("e")
+    json = flask.request.args.get("json")  # Return the data as json
     search_db = flask.request.args.get("database")
 
-    if embed:
+    table_data = bd.Database(search_db or config["target"]).search(q, limit=100)
+
+    if json:
+        MAPPING = {"reference product": "referenceProduct"}
+        FIELDS = {"name", "id", "location", "unit", "reference product"}
+
+        payload = [
+            {MAPPING.get(key, key): ds.get(key) for key in FIELDS} for ds in table_data
+        ]
+        return flask.jsonify(payload)
+    elif embed:
         source = bd.get_activity(id=int(flask.request.args.get("source")))
         return flask.render_template(
             "search-embedded.html",
             source=source,
-            table_data=bd.Database(search_db or t).search(q, limit=100),
+            table_data=table_data,  # bd.Database(search_db or t).search(q, limit=100),
         )
     else:
         return flask.render_template(
             "index.html",
-            project=proj,
-            source=s,
-            target=t,
-            file_number=sum(1 for obj in files if obj["enabled"]),
-            proxy=proxy,
-            user=auth.current_user(),
+            config=config,
             title="bw_matchbox Search Result",
-            table_data=bd.Database(search_db).search(q, limit=100),
+            table_data=table_data,  # =bd.Database(search_db).search(q, limit=100),
             query_string=q,
-            database=s,
         )
 
 
 @matchbox_app.route("/add-attribute/<id>", methods=["GET"])
 @auth.login_required
 def add_attribute(id):
-    context = get_context()
-    proj, s, t, proxy = context
-    bd.projects.set_current(proj)
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    if config["role"] != "editors":
+        flask.abort(403)
+
     node = bd.get_node(id=id)
 
     attr = flask.request.args.get("attr")
@@ -517,17 +407,51 @@ def get_authors(authors):
         return ", ".join({obj.get("name", "Unknown") for obj in authors.values()})
 
 
+@matchbox_app.route("/delete-proxy/<id>", methods=["GET"])
+@auth.login_required
+def delete_proxy(id):
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    if config["role"] != "editors":
+        flask.abort(403)
+
+    node = bd.get_node(id=id)
+    if not node["database"] == config["proxy"]:
+        raise ValueError(
+            "Not a proxy dataset in the proxy database {}".format(config["proxy"])
+        )
+
+    if node.get("original_id"):
+        reference = bd.get_node(id=node["original_id"])
+        for field in ("proxy_id", "matched", "match_type"):
+            if field in reference:
+                del reference[field]
+        reference.save()
+    else:
+        reference = None
+
+    node.delete()
+
+    if reference is None:
+        return flask.redirect(flask.url_for("index"))
+    else:
+        return flask.redirect(flask.url_for("process_detail", id=reference.id))
+
+
 @matchbox_app.route("/process/<id>", methods=["GET"])
 @auth.login_required
 def process_detail(id):
-    context = get_context()
-    if isinstance(context, flask.Response):
-        return context
-    proj, s, t, proxy = context
-    files = get_files()
-    bd.projects.set_current(proj)
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    try:
+        node = bd.get_node(id=id)
+    except bd.errors.UnknownObject:
+        return flask.redirect(flask.url_for("index"))
 
-    node = bd.get_node(id=id)
     proxy_node = bd.get_node(id=node["proxy_id"]) if node.get("proxy_id") else None
     reference_node = (
         bd.get_node(id=node["original_id"]) if node.get("original_id") else None
@@ -539,20 +463,17 @@ def process_detail(id):
     same_name_count = same_name.count()
     technosphere = sorted(node.technosphere(), key=lambda x: x.input["name"])
     biosphere = sorted(node.biosphere(), key=lambda x: x.input["name"])
+    is_proxy = node["database"] == config["proxy"]
 
     return flask.render_template(
         "process_detail.html",
         title="bw_matchbox Detail Page",
         ds=node,
+        config=config,
         authors=get_authors(node.get("authors")),
-        project=proj,
-        proxy=proxy,
         proxy_node=proxy_node,
         reference_node=reference_node,
-        source=s,
-        target=t,
-        file_number=sum(1 for obj in files if obj["enabled"]),
-        user=auth.current_user(),
+        source=config["source"],
         same_name_count=same_name_count,
         same_name=same_name,
         technosphere=technosphere,
@@ -560,15 +481,19 @@ def process_detail(id):
         match_type=matchbox_app.config["mb_match_types"].get(node.get("match_type")),
         total_consumers=len(node.upstream()),
         consumers=list(node.upstream())[:50],
+        is_proxy=is_proxy,
     )
 
 
 @matchbox_app.route("/multi-match/<id>", methods=["GET"])
 @auth.login_required
 def multi_match(id):
-    context = get_context()
-    proj, s, t, proxy = context
-    bd.projects.set_current(proj)
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    if config["role"] != "editors":
+        flask.abort(403)
 
     node = bd.get_node(id=id)
     node["matched"] = True
@@ -590,29 +515,33 @@ def multi_match(id):
 @matchbox_app.route("/match/<source>", methods=["GET"])
 @auth.login_required
 def match(source):
-    context = get_context()
-    if isinstance(context, flask.Response):
-        return context
-    proj, s, t, proxy = context
-    files = get_files()
-    bd.projects.set_current(proj)
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    if config["role"] != "editors":
+        flask.abort(403)
 
     node = bd.get_node(id=source)
 
-    matches = bd.Database(t).search(node["name"] + " " + node.get("location", ""))
+    matches = bd.Database(config["target"]).search(
+        node["name"] + " " + node.get("location", "")
+    )
+
+    MAPPING = {"reference product": "referenceProduct"}
+    FIELDS = {"name", "id", "location", "unit", "reference product"}
+    matches_payload = [
+        {MAPPING.get(key, key): ds.get(key) for key in FIELDS} for ds in matches
+    ]
 
     return flask.render_template(
         "match.html",
         title="Database Matching Page",
+        config=config,
         ds=node,
-        project=proj,
-        proxy=proxy,
-        source=s,
-        target=t,
-        file_number=sum(1 for obj in files if obj["enabled"]),
-        user=auth.current_user(),
         query_string=node["name"] + " " + node.get("location", ""),
         matches=matches,
+        matches_json=json.dumps(matches_payload),
     )
 
 
@@ -639,13 +568,9 @@ def check_similar(node, candidates):
 @matchbox_app.route("/compare/<source>/<target>", methods=["GET"])
 @auth.login_required
 def compare(source, target):
-    context = get_context()
-    if isinstance(context, flask.Response):
-        return context
-    proj, s, t, proxy = context
-
-    files = get_files()
-    bd.projects.set_current(proj)
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
 
     source = bd.get_node(id=int(source))
     target = bd.get_node(id=int(target))
@@ -659,6 +584,8 @@ def compare(source, target):
             "location": exc.input["location"],
             "input_id": exc.input.id,
             "url": flask.url_for("process_detail", id=exc.input.id),
+            # Issue #59: Name cell with check icon for 'matched' rows
+            "matched": bool(exc.input.get("matched")),
         }
         for exc in source.technosphere()
     ]
@@ -688,16 +615,15 @@ def compare(source, target):
         # Modifies in place
         check_similar(obj, source_technosphere)
 
+    base_url = flask.request.base_url.replace(flask.request.path, "")
+    print(base_url)
+
     return flask.render_template(
         "compare.html",
+        config=config,
         title="Database Comparison Page",
-        project=proj,
-        proxy=proxy,
-        source=s,
-        target=t,
+        proxy=config["proxy"],
         target_json=json.dumps(target_itself),
-        user=auth.current_user(),
-        file_number=sum(1 for obj in files if obj["enabled"]),
         source_node=source,
         source_data_json=json.dumps(source_technosphere),
         source_biosphere_number=len(source.biosphere()),
@@ -705,18 +631,17 @@ def compare(source, target):
         target_data_json=json.dumps(target_technosphere),
         target_biosphere_number=len(target.biosphere()),
         match_types=matchbox_app.config["mb_match_types"],
+        base_url=base_url,
     )
 
 
 @matchbox_app.route("/expand/<id>/<scale>/", methods=["GET"])
 @auth.login_required
 def expand(id, scale):
-    context = get_context()
-    if isinstance(context, flask.Response):
-        return context
-    proj, s, t, proxy = context
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
 
-    bd.projects.set_current(proj)
     node = bd.get_activity(id=int(id))
     scale = float(scale)
 
@@ -742,12 +667,17 @@ def expand(id, scale):
 @matchbox_app.route("/create-proxy/", methods=["POST"])
 @auth.login_required
 def create_proxy():
+    config = get_config()
+    if isinstance(config, flask.Response):
+        return config
+
+    if config["role"] != "editors":
+        flask.abort(403)
+
     content = flask.request.get_json()
-    proj, s, t, proxy = get_context()
-    bd.projects.set_current(proj)
 
     source = bd.get_activity(id=content["source"])
-    process = bd.Database(proxy).new_activity(
+    process = bd.Database(config["proxy"]).new_activity(
         name=content["name"],
         code=uuid.uuid4().hex,
         comment=content["comment"],
@@ -756,7 +686,7 @@ def create_proxy():
         original_name=source["name"],
         original_id=source.id,
         match_type=content.get("match_type", "0"),
-        **{"reference product": source.get("reference product")}
+        **{"reference product": source.get("reference product")},
     )
     process.save()
 
